// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace Esmf
{
    public class Variable1Dimensional<D1, T> : IVariable1Dimensional<D1, T>, IParameter1Dimensional<D1, T>
    {
        private JaggedArrayWrapper<T> _values;
        private Enum2IntConversionDelegate<D1> _enumConversionDelegate;
        private ModelFields _parent;

        public Variable1Dimensional(ModelFields parent)
        {
            _values = new JaggedArrayWrapper<T>(16);

            _enumConversionDelegate = Enum2IntConversionSingleton<D1>.Instance.Delegate;

            _parent = parent;
        }

        public Variable1Dimensional(ModelFields parent, ParameterValue1Dimensional<T> values)
        {
            _parent = parent;

            _values = new JaggedArrayWrapper<T>(values.Length);

            _enumConversionDelegate = Enum2IntConversionSingleton<D1>.Instance.Delegate;

            values.CopyTo(_values);
        }

        [Conditional("FUNDNANCHECK")]
        private void CheckForValidValue(T v)
        {
            if (v.GetType() == typeof(double))
            {
                double d = ((double)((object)v));

                if (double.IsNaN(d))
                    throw new ArgumentOutOfRangeException("NaN is not allowed as a value");
                else if (double.IsInfinity(d))
                    throw new ArgumentOutOfRangeException("Infinity is not allowed as a value");
            }
        }

        public T this[D1 index]
        {
            get
            {
                return _values[_enumConversionDelegate(index) - 1];
            }
            set
            {
                CheckForValidValue(value);

                _values[_enumConversionDelegate(index) - 1] = value;
            }
        }

        #region IParameter1Dimensional<D1,T> Members

        T IParameter1Dimensional<D1, T>.this[D1 index]
        {
            get
            {
                return this[index];

            }
        }

        IEnumerable<Parameter1DimensionalMember<D1, T>> IParameter1Dimensional<D1, T>.GetEnumerator()
        {
            D1[] dimensionValues = (D1[])Enum.GetValues(typeof(D1));
            int dimensionCount = dimensionValues.Length;
            for (int i = 0; i < dimensionCount; i++)
            {
                yield return new Parameter1DimensionalMember<D1, T>(dimensionValues[i], _values[i], _parent.AttachedDimensions);
            }
        }

        #endregion

        #region IVariable1Dimensional<D1,T> Members

        T IVariable1Dimensional<D1, T>.this[D1 index]
        {
            get
            {
                return this[index];
            }
            set
            {
                this[index] = value;
            }
        }

        IEnumerable<Parameter1DimensionalMember<D1, T>> IVariable1Dimensional<D1, T>.GetEnumerator()
        {
            return (this as IParameter1Dimensional<D1, T>).GetEnumerator();
        }

        #endregion

    }
}
