// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Reflection.Emit;
using System.Diagnostics;

namespace Esmf
{
    public class Variable2DimensionalTime<D2, T> : IVariable2Dimensional<Timestep, D2, T>, IParameter2Dimensional<Timestep, D2, T>
    {
        private JaggedArrayWrapper2<T> _values;
        private Enum2IntConversionDelegate<D2> _enumConversionDelegate;
        private ModelFields _parent;
        private const int _NYears = 1050;

        public Variable2DimensionalTime(ModelFields parent)
        {
            // TODO Change this to properly investigate the dimensions
            _values = new JaggedArrayWrapper2<T>(_NYears, 16);

            _enumConversionDelegate = Enum2IntConversionSingleton<D2>.Instance.Delegate;

            _parent = parent;
        }

        public Variable2DimensionalTime(ModelFields parent, ParameterValue2Dimensional<T> values)
        {
            _parent = parent;

            _enumConversionDelegate = Enum2IntConversionSingleton<D2>.Instance.Delegate;

            _values = new JaggedArrayWrapper2<T>(values.Length0, values.Length1);

            values.CopyTo(_values);
        }

        [Conditional("FUNDNANCHECK")]
        private void CheckForValidValue(T v)
        {
            if (v.GetType() == typeof(double))
            {
                double d = ((double)((object)v));

                if (double.IsNaN(d))
                    throw new ArgumentOutOfRangeException("NaN is not allowed as a value");
                else if (double.IsInfinity(d))
                    throw new ArgumentOutOfRangeException("Infinity is not allowed as a value");
            }
        }

        public T this[Timestep index1, D2 index2]
        {
            get
            {
                return _values[index1.Value, _enumConversionDelegate(index2) - 1];
            }
            set
            {
                CheckForValidValue(value);
                _values[index1.Value, _enumConversionDelegate(index2) - 1] = value;
            }
        }


        #region IVariable2Dimensional<Timestep,D2,T> Members

        T IVariable2Dimensional<Timestep, D2, T>.this[Timestep index1, D2 index2]
        {
            get
            {
                return this[index1, index2];
            }
            set
            {
                this[index1, index2] = value;
            }
        }

        IEnumerable<Parameter2DimensionalMember<Timestep, D2, T>> IVariable2Dimensional<Timestep, D2, T>.GetEnumerator()
        {
            return (this as IParameter2Dimensional<Timestep, D2, T>).GetEnumerator();
        }

        #endregion

        #region IParameter2Dimensional<Timestep,D2,T> Members

        T IParameter2Dimensional<Timestep, D2, T>.this[Timestep index1, D2 index2]
        {
            get
            {
                return this[index1, index2];
            }
        }

        IEnumerable<Parameter2DimensionalMember<Timestep, D2, T>> IParameter2Dimensional<Timestep, D2, T>.GetEnumerator()
        {
            D2[] values = (D2[]) Enum.GetValues(typeof(D2));
            for (int i = 0; i < _NYears; i++)
            {
                for (int l = 0; l < 16; l++)
                {
                    yield return new Parameter2DimensionalMember<Timestep,D2,T>(Timestep.FromSimulationYear(i), values[l],_values[i,l], _parent.AttachedDimensions);
                }
            }
        }

        #endregion

    }
}
