﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund
{

    // Declining discount rate according to the UK green book
    public class GreenbookDiscounting : WeightingScheme
    {
        private int m_baseYear;

        public override void CalculateWeights(ModelOutput ModelOutput)
        {
            double i_discountFactor;
            double i_discountRate;


            i_discountFactor = 1;

            for (int i = 0; i <= LegacyConstants.NYear - m_baseYear; i++)
            {
                for (int l = 1; l <= LegacyConstants.NoReg; l++)
                    AddWeight(i + m_baseYear, l, i_discountFactor);

                if (i < 30)
                    i_discountRate = 0.035;
                else if (i < 75)
                    i_discountRate = 0.03;
                else if (i < 125)
                    i_discountRate = 0.025;
                else if (i < 200)
                    i_discountRate = 0.02;
                else if (i < 300)
                    i_discountRate = 0.015;
                else
                    i_discountRate = 0.01;

                i_discountFactor = i_discountFactor / (1.00 + i_discountRate);
            }
        }

        public GreenbookDiscounting(int BaseYear)
            : base()
        {
            m_baseYear = BaseYear;
        }

        protected override string GetName()
        {
            return "greenbook discounting (base year: " + m_baseYear.ToString() + ")";
        }

    }
}