﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.6
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Esmf;

namespace Fund.Components.TerrestrialBiosphere
{

    public interface ITerrestrialBiosphereState
    {
        /// <summary>
        /// CO2 emissions from the terrestrial biosphere
        /// </summary>
        IVariable1Dimensional<Timestep, double> TerrestrialCO2 { get; }

        /// <summary>
        /// Stock of CO2 in the terrestrial biosphere
        /// </summary>
        IVariable1Dimensional<Timestep, double> TerrCO2Stock { get; }

        IParameter1Dimensional<Timestep, double> temp { get; }

        double TerrCO2Sens { get; }
        double TerrCO2Stock0 { get; }

    }

    public class TerrestrialBiosphereComponent
    {
        public void Run(Clock clock, ITerrestrialBiosphereState state)
        {
            var s = state;
            var t = clock.Current;

            //terrestrial biosphere
            //TerrSO2Sens is 2600 in best guess, with 600 and 7500 as upper and lower bounds
            //TerrCO2StockInit is 1900000
            //TerrCO2Stock should be initialised at TerrCO2StockInit
            //0.963703 should replaced with temp for t = 60
            //should we introduce a constant Future = 60?

            if (clock.IsFirstTimestep)
            {
                s.TerrCO2Stock[t] = s.TerrCO2Stock0;
            }
            else
            {
                if (t > Timestep.FromYear(2010))
                {
                    s.TerrestrialCO2[t] = (s.temp[t - 1] - s.temp[Timestep.FromYear(2010)]) * s.TerrCO2Sens * s.TerrCO2Stock[t-1] / s.TerrCO2Stock[Timestep.FromYear(2010)];
                }
                else
                    s.TerrestrialCO2[t] = 0;

                s.TerrCO2Stock[t] = Math.Max(s.TerrCO2Stock[t - 1] - s.TerrestrialCO2[t], 0.0);
            }
        }
    }
}
